// -*- C++ -*-
//===----------------------------------------------------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

export namespace std {
  // [fp.style], floating-point type properties
  using std::float_denorm_style;
  using std::float_round_style;

  // [numeric.limits], class template numeric_­limits
  using std::numeric_limits;
} // namespace std
